@@ -322,10 +322,11 @@ class _Intrinsic(ReduceMixin):
 
     __uuid = None
 
-    def __init__(self, name, defn, **kwargs):
+    def __init__(self, name, defn, prefer_literal=False, **kwargs):
         self._ctor_kwargs = kwargs
         self._name = name
         self._defn = defn
+        self._prefer_literal = prefer_literal
         functools.update_wrapper(self, defn)
 
     @property
@@ -354,7 +355,8 @@ class _Intrinsic(ReduceMixin):
                                                  infer_global)
 
         template = make_intrinsic_template(self, self._defn, self._name,
-                                           self._ctor_kwargs)
+                                           prefer_literal=self._prefer_literal,
+                                           kwargs=self._ctor_kwargs)
         infer(template)
         infer_global(self, types.Function(template))
 
