@@ -9,7 +9,7 @@ import inspect
 import os.path
 from collections import namedtuple
 from collections.abc import Sequence
-from types import MethodType, FunctionType
+from types import MethodType, FunctionType, MappingProxyType
 
 import numba
 from numba.core import types, utils, targetconfig
@@ -999,15 +999,18 @@ class _IntrinsicTemplate(_TemplateTargetHelperMixin, AbstractTemplate):
         return info
 
 
-def make_intrinsic_template(handle, defn, name, kwargs):
+def make_intrinsic_template(handle, defn, name, *, prefer_literal=False,
+                            kwargs=None):
     """
     Make a template class for a intrinsic handle *handle* defined by the
     function *defn*.  The *name* is used for naming the new template class.
     """
+    kwargs = MappingProxyType({} if kwargs is None else kwargs)
     base = _IntrinsicTemplate
     name = "_IntrinsicTemplate_%s" % (name)
     dct = dict(key=handle, _definition_func=staticmethod(defn),
-               _impl_cache={}, _overload_cache={}, metadata=kwargs)
+               _impl_cache={}, _overload_cache={},
+               prefer_literal=prefer_literal, metadata=kwargs)
     return type(base)(name, (base,), dct)
 
 
@@ -1118,6 +1121,8 @@ class _OverloadMethodTemplate(_OverloadAttributeTemplate):
 
         if isinstance(typ, types.TypeRef):
             assert typ == self.key
+        elif isinstance(typ, types.Callable):
+            assert typ == self.key
         else:
             assert isinstance(typ, self.key)
 
@@ -1138,6 +1143,22 @@ class _OverloadMethodTemplate(_OverloadAttributeTemplate):
                 if sig is not None:
                     return sig.as_method()
 
+            def get_template_info(self):
+                basepath = os.path.dirname(os.path.dirname(numba.__file__))
+                impl = self._overload_func
+                code, firstlineno, path = self.get_source_code_info(impl)
+                sig = str(utils.pysignature(impl))
+                info = {
+                    'kind': "overload_method",
+                    'name': getattr(impl, '__qualname__', impl.__name__),
+                    'sig': sig,
+                    'filename': utils.safe_relpath(path, start=basepath),
+                    'lines': (firstlineno, firstlineno + len(code) - 1),
+                    'docstring': impl.__doc__
+                }
+
+                return info
+
         return types.BoundFunction(MethodTemplate, typ)
 
 
