@@ -4,19 +4,14 @@ from itertools import permutations
 import numpy as np
 
 import unittest
-from numba.core.compiler import compile_isolated, Flags
 from numba import jit, njit, from_dtype, typeof
-from numba.np.numpy_support import numpy_version
 from numba.core.errors import TypingError
 from numba.core import types, errors
-from numba.tests.support import (TestCase, MemoryLeakMixin, CompilationCache,
-                                 tag)
+from numba.tests.support import TestCase, MemoryLeakMixin
 
-enable_pyobj_flags = Flags()
-enable_pyobj_flags.enable_pyobject = True
+enable_pyobj_flags = {'forceobj': True}
 
-no_pyobj_flags = Flags()
-no_pyobj_flags.nrt = True
+no_pyobj_flags = {'_nrt': True, 'nopython': True}
 
 
 def from_generic(pyfuncs_to_use):
@@ -41,10 +36,11 @@ def from_generic(pyfuncs_to_use):
     return decorator
 
 
+@njit
 def array_reshape(arr, newshape):
     return arr.reshape(newshape)
 
-
+@njit
 def numpy_array_reshape(arr, newshape):
     return np.reshape(arr, newshape)
 
@@ -88,22 +84,28 @@ def transpose_array(a):
 def numpy_transpose_array(a):
     return np.transpose(a)
 
+
+@njit
 def numpy_transpose_array_axes_kwarg(arr, axes):
     return np.transpose(arr, axes=axes)
 
 
+@njit
 def numpy_transpose_array_axes_kwarg_copy(arr, axes):
     return np.transpose(arr, axes=axes).copy()
 
 
+@njit
 def array_transpose_axes(arr, axes):
     return arr.transpose(axes)
 
 
+@njit
 def array_transpose_axes_copy(arr, axes):
     return arr.transpose(axes).copy()
 
 
+@njit
 def transpose_issue_4708(m, n):
     r1 = np.reshape(np.arange(m * n * 3), (m, 3, n))
     r2 = np.reshape(np.arange(n * 3), (n, 3))
@@ -180,26 +182,23 @@ def numpy_flatnonzero(a):
 def numpy_argwhere(a):
     return np.argwhere(a)
 
+def numpy_resize(a, new_shape):
+    return np.resize(a, new_shape)
+
 
 class TestArrayManipulation(MemoryLeakMixin, TestCase):
     """
     Check shape-changing operations on arrays.
     """
-
-    def setUp(self):
-        super(TestArrayManipulation, self).setUp()
-        self.ccache = CompilationCache()
-
     def test_array_reshape(self):
         pyfuncs_to_use = [array_reshape, numpy_array_reshape]
 
         def generic_run(pyfunc, arr, shape):
-            cres = compile_isolated(pyfunc, (typeof(arr), typeof(shape)))
-            return cres.entry_point(arr, shape)
+            return pyfunc(arr, shape)
 
         @from_generic(pyfuncs_to_use)
         def check(pyfunc, arr, shape):
-            expected = pyfunc(arr, shape)
+            expected = pyfunc.py_func(arr, shape)
             self.memory_leak_setup()
             got = generic_run(pyfunc, arr, shape)
             self.assertPreciseEqual(got, expected)
@@ -308,14 +307,10 @@ class TestArrayManipulation(MemoryLeakMixin, TestCase):
                           array_transpose_axes,
                           array_transpose_axes_copy]
 
-        def run(pyfunc, arr, axes):
-            cres = self.ccache.compile(pyfunc, (typeof(arr), typeof(axes)))
-            return cres.entry_point(arr, axes)
-
         @from_generic(pyfuncs_to_use)
         def check(pyfunc, arr, axes):
-            expected = pyfunc(arr, axes)
-            got = run(pyfunc, arr, axes)
+            expected = pyfunc.py_func(arr, axes)
+            got = pyfunc(arr, axes)
             self.assertPreciseEqual(got, expected)
             self.assertEqual(got.flags.f_contiguous,
                              expected.flags.f_contiguous)
@@ -325,21 +320,21 @@ class TestArrayManipulation(MemoryLeakMixin, TestCase):
         @from_generic(pyfuncs_to_use)
         def check_err_axis_repeated(pyfunc, arr, axes):
             with self.assertRaises(ValueError) as raises:
-                run(pyfunc, arr, axes)
+                pyfunc(arr, axes)
             self.assertEqual(str(raises.exception),
                              "repeated axis in transpose")
 
         @from_generic(pyfuncs_to_use)
         def check_err_axis_oob(pyfunc, arr, axes):
             with self.assertRaises(ValueError) as raises:
-                run(pyfunc, arr, axes)
+                pyfunc(arr, axes)
             self.assertEqual(str(raises.exception),
                              "axis is out of bounds for array of given dimension")
 
         @from_generic(pyfuncs_to_use)
         def check_err_invalid_args(pyfunc, arr, axes):
             with self.assertRaises((TypeError, TypingError)):
-                run(pyfunc, arr, axes)
+                pyfunc(arr, axes)
 
         arrs = [np.arange(24),
                 np.arange(24).reshape(4, 6),
@@ -359,8 +354,8 @@ class TestArrayManipulation(MemoryLeakMixin, TestCase):
 
         @from_generic([transpose_issue_4708])
         def check_issue_4708(pyfunc, m, n):
-            expected = pyfunc(m, n)
-            got = njit(pyfunc)(m, n)
+            expected = pyfunc.py_func(m, n)
+            got = pyfunc(m, n)
             # values in arrays are equals,
             # but stronger assertions not hold (layout and strides equality)
             np.testing.assert_equal(got, expected)
@@ -396,17 +391,64 @@ class TestArrayManipulation(MemoryLeakMixin, TestCase):
         self.assertIn("np.transpose does not accept tuples",
                         str(e.exception))
 
+    def test_numpy_resize_basic(self):
+        pyfunc = numpy_resize
+        cfunc = njit(pyfunc)
+        def inputs():
+            # Taken from https://github.com/numpy/numpy/blob/f0b2fca91a1f5f50ff696895072f6fe9e69c1466/numpy/core/tests/test_numeric.py#L24-L64  noqa: E501
+            yield np.array([[1, 2], [3, 4]]), (2, 4)
+            yield np.array([[1, 2], [3, 4]]), (4, 2)
+            yield np.array([[1, 2], [3, 4]]), (4, 3)
+            yield np.array([[1, 2], [3, 4]]), (0,)
+            yield np.array([[1, 2], [3, 4]]), (0, 2)
+            yield np.array([[1, 2], [3, 4]]), (2, 0)
+            yield np.zeros(0, dtype = float), (2, 1)
+            # other
+            yield np.array([[1, 2], [3, 4]]), (4,)
+            yield np.array([[1, 2], [3, 4]]), 4
+            yield np.zeros((1, 3), dtype = int), (2, 1)
+            yield np.array([], dtype = float), (4, 2)
+            yield [0, 1, 2, 3], (2, 3)
+            yield 4, (2, 3)
+
+        for a, new_shape in inputs():
+            self.assertPreciseEqual(pyfunc(a, new_shape), cfunc(a, new_shape))
+
+    def test_numpy_resize_exception(self):
+        # Exceptions leak references
+        self.disable_leak_check()
+
+        cfunc = njit(numpy_resize)
+
+        with self.assertRaises(TypingError) as raises:
+            cfunc("abc", (2, 3))
+        self.assertIn(('The argument "a" must be array-like'),
+                      str(raises.exception))
+
+        with self.assertRaises(TypingError) as raises:
+            cfunc(np.array([[0,1],[2,3]]), "abc")
+        self.assertIn(('The argument "new_shape" must be an integer or '
+                       'a tuple of integers'),
+                      str(raises.exception))
+
+        with self.assertRaises(ValueError) as raises:
+            cfunc(np.array([[0,1],[2,3]]), (-2, 3))
+        self.assertIn(('All elements of `new_shape` must be non-negative'),
+                      str(raises.exception))
+
+        with self.assertRaises(ValueError) as raises:
+            cfunc(np.array([[0,1],[2,3]]), -4)
+        self.assertIn(('All elements of `new_shape` must be non-negative'),
+                      str(raises.exception))
+
     def test_expand_dims(self):
         pyfunc = expand_dims
-
-        def run(arr, axis):
-            cres = self.ccache.compile(pyfunc, (typeof(arr), typeof(axis)))
-            return cres.entry_point(arr, axis)
+        cfunc = njit(pyfunc)
 
         def check(arr, axis):
             expected = pyfunc(arr, axis)
             self.memory_leak_setup()
-            got = run(arr, axis)
+            got = cfunc(arr, axis)
             self.assertPreciseEqual(got, expected)
             del got
             self.memory_leak_teardown()
@@ -504,12 +546,11 @@ class TestArrayManipulation(MemoryLeakMixin, TestCase):
         self.check_atleast_nd(pyfunc, cfunc)
 
     def check_as_strided(self, pyfunc):
-        def run(arr):
-            cres = self.ccache.compile(pyfunc, (typeof(arr),))
-            return cres.entry_point(arr)
+        cfunc = njit(pyfunc)
+
         def check(arr):
             expected = pyfunc(arr)
-            got = run(arr)
+            got = cfunc(arr)
             self.assertPreciseEqual(got, expected)
 
         arr = np.arange(24)
@@ -637,8 +678,7 @@ class TestArrayManipulation(MemoryLeakMixin, TestCase):
             arraytype1 = arraytype1.copy(layout='A')
 
         self.assertEqual(arraytype1.layout, layout)
-        cr = compile_isolated(pyfunc, (arraytype1,), flags=flags)
-        cfunc = cr.entry_point
+        cfunc = jit((arraytype1,), **flags)(pyfunc)
 
         expected = pyfunc(a)
         got = cfunc(a)
@@ -654,8 +694,7 @@ class TestArrayManipulation(MemoryLeakMixin, TestCase):
             # compile
             arraytype1 = typeof(a)
             self.assertEqual(arraytype1.layout, assume_layout)
-            cr = compile_isolated(pyfunc, (arraytype1,), flags=flags)
-            cfunc = cr.entry_point
+            cfunc = jit((arraytype1,), **flags)(pyfunc)
 
             expected = pyfunc(a)
             got = cfunc(a)
@@ -689,8 +728,7 @@ class TestArrayManipulation(MemoryLeakMixin, TestCase):
 
         pyfunc = ravel_array_size
         arraytype1 = typeof(a)
-        cr = compile_isolated(pyfunc, (arraytype1,), flags=flags)
-        cfunc = cr.entry_point
+        cfunc = jit((arraytype1,), **flags)(pyfunc)
 
         expected = pyfunc(a)
         got = cfunc(a)
@@ -708,8 +746,7 @@ class TestArrayManipulation(MemoryLeakMixin, TestCase):
             a = np.arange(9).reshape(3, 3)
 
             arraytype1 = typeof(a)
-            cr = compile_isolated(pyfunc, (arraytype1,), flags=flags)
-            cfunc = cr.entry_point
+            cfunc = jit((arraytype1,), **flags)(pyfunc)
 
             expected = pyfunc(a)
             got = cfunc(a)
@@ -725,8 +762,7 @@ class TestArrayManipulation(MemoryLeakMixin, TestCase):
 
         pyfunc = squeeze_array
         arraytype1 = typeof(a)
-        cr = compile_isolated(pyfunc, (arraytype1,), flags=flags)
-        cfunc = cr.entry_point
+        cfunc = jit((arraytype1,), **flags)(pyfunc)
 
         expected = pyfunc(a)
         got = cfunc(a)
@@ -787,14 +823,12 @@ class TestArrayManipulation(MemoryLeakMixin, TestCase):
             arraytype1 = from_dtype(np.dtype([('x', np.int32),
                                               ('y', np.int32)]))
             arraytype2 = types.Array(types.int32, 2, 'C')
-            compile_isolated(bad_index, (arraytype1, arraytype2),
-                             flags=no_pyobj_flags)
+            njit((arraytype1, arraytype2))(bad_index)
         self.assertIn('Unsupported array index type', str(raises.exception))
 
     def test_bad_float_index_npm(self):
         with self.assertTypingError() as raises:
-            compile_isolated(bad_float_index,
-                             (types.Array(types.float64, 2, 'C'),))
+            njit((types.Array(types.float64, 2, 'C'),))(bad_float_index)
         self.assertIn('Unsupported array index type float64',
                       str(raises.exception))
 
